//
//  O3D_Camera.h
//  Objective3D
//
//  Created by charlie on 5/28/09.
//  Copyright 2009 One More Muse. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import <OpenGL/OpenGL.h>
#import <OpenGL/glu.h>
#import "O3D_Vector3.h"
#import "O3D_Matrix.h"
#import "O3D_Quaternion.h"
#import "O3D_Util.h"

/// Camera object handling perspective, view rotation and transformation

@interface O3D_Camera : NSObject {
	/// X, Y, Z coordinates for camera. The inverse of this is multiplied by the modelview matrix
	float x, y, z; 
	/// Conveience properties used in movement / rotation methods that don't accept arguments
	float movementSpeed, rotationSpeed;
	
	/// Quaternion representing camera rotation
	O3D_Quaternion *rotation;
	/// Perspective matrix; normally sent once immediately after camera initialization
	O3D_Matrix *perspective;
	/// location matrix; generated from the inverse of the x, y, and z propeties
	O3D_Matrix *location;
}

@property (nonatomic) float x, y, z, movementSpeed, rotationSpeed;
@property (nonatomic, retain) O3D_Quaternion *rotation;

/// Called after loading the identity for the modelview; this performs multiplications for camera location / rotation
- (void) load;

/// Move the receiver left (strafe) by a specified amount
- (void) left:(float)amount;
/// Move the receiver left (strafe) by the movementSpeed property
- (void) left;
/// Move the receiver right (strafe) by a specified amount
- (void) right:(float)amount;
/// Move the receiver right (strafe) by the movementSpeed property
- (void) right;
/// Move the receiver up by a specified amount
- (void) up:(float)amount;
/// Move the receiver up by the movementSpeed property
- (void) up;
/// Move the receiver down by a specified amount
- (void) down:(float)amount;
/// Move the receiver down by the movementSpeed property
- (void) down;
/// Move the receiver along the Z axis by the specified amount
- (void) forward:(float)amount;
/// Move the receiver along the Z axis by the movementSpeed property
- (void) forward;
/// Move the receiver along the Z axis by the specified amount
- (void) backward:(float)amount;
/// Move the receiver along the Z axis by the movementSpeed property
- (void) backward;

/// Move along the X, Y and Z axis simultaneously
- (void) moveX:(float)ax Y:(float)ay Z:(float)az;

/// Rotate on the Y axis by the specified amount
- (void) pan:(float)amount;
/// Rotate left the Y axis by the rotationSpeed property
- (void) panLeft;
/// Rotate right the Y axis by the rotationSpeed property
- (void) panRight;

/// Rotate around the X axis by the specified amount
- (void) tilt:(float)amount;
/// Rotate backwards around the X axis by the rotationSpeed property
- (void) tiltUp;
/// Rotate forwards around the X axis by the rotationSpeed property
- (void) tiltDown;

/// Rotate by an amount on a specified axis 
- (void) rotateByAngle:(float)angle onAxis:(O3D_Vector3 *)axis;
/// Set the camera perspective matrix
- (void) setPerspectiveWithFOV:(float)fov aspect:(float)aspect zNear:(float)zNear zFar:(float)zFar;
/// Apply the camera perspective
- (void) applyPerspective;

@end
